/*
 * Copyright (C) 2007 Andre Beckedorf <evilJazz _AT_ katastrophos _DOT_ net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include <qpe/qpeapplication.h>
#include <qpe/qpemenubar.h>
#include <qpe/global.h>

#include <qlayout.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qslider.h>
#include <qfile.h>
#include <qmessagebox.h>

#include "wm8750mixer.h"

WM8750Mixer::WM8750Mixer()
{
	setCaption("Enhanced Mixer for WM8750");
	
	// Menubar stuff
    QPopupMenu *help = new QPopupMenu(this);
    help->insertItem(tr("&About..."), this, SLOT(about()));
    
    QMenuBar *menu = new QMenuBar(this);
    menu->insertItem("&Help", help);
    menu->setSeparator(QMenuBar::InWindowsStyle);
    
    updateWidgets();
    updateValues();
}

WM8750Mixer::~WM8750Mixer()
{
}

void WM8750Mixer::closeEvent(QCloseEvent *e)
{
	// Invoke external script to save settings:
	pid_t pid = fork();
	if (pid <= 0)
	{
		// force super-user rights from parent to child...
		setuid(0);
		setgid(0);
		
		const char *args[3];
		args[0] = "wm8750";
		args[1] = "save";
		args[2] = NULL;
		
		execvp(args[0], (char* const*)args);
		_exit(1); // return error if we're still here...
	}
	else
	{
		wait(&pid);
	}

	e->accept();
	return;
}

void WM8750Mixer::updateWidgets()
{
	QVBoxLayout *vl = new QVBoxLayout(this, 8);
	vl->setAutoAdd(true);

	QHBox *hboxOutput = new QHBox(this);
	new QLabel("Output:", hboxOutput);
	
	pbOutputAuto = new QPushButton("Autodetect", hboxOutput);
	pbOutputAuto->setToggleButton(true);
	connect(pbOutputAuto, SIGNAL(toggled(bool)), this, SLOT(pbOutputAutoToggled(bool)));
	
	pbOutputInternalSpeaker = new QPushButton("Internal", hboxOutput);
	pbOutputInternalSpeaker->setToggleButton(true);
	connect(pbOutputInternalSpeaker, SIGNAL(toggled(bool)), this, SLOT(pbOutputInternalSpeakerToggled(bool)));
	
	pbOutputHeadphone = new QPushButton("Headphone", hboxOutput);
	pbOutputHeadphone->setToggleButton(true);
	connect(pbOutputHeadphone, SIGNAL(toggled(bool)), this, SLOT(pbOutputHeadphoneToggled(bool)));
	
	QHBox *hboxEqualizer = new QHBox(this);
	hboxEqualizer->setSpacing(8);
	
	// Bass
	QVGroupBox *gbBass = new QVGroupBox("Bass", hboxEqualizer);
	gbBass->setAlignment(QGroupBox::AlignHCenter);
	
	cbBassEnabled = new QCheckBox("Enabled", gbBass);
	connect(cbBassEnabled, SIGNAL(toggled(bool)), this, SLOT(cbBassEnabledToggled(bool)));
	
	QHBox *hboxSliderBass = new QHBox(gbBass);
	
	sliderBass = new QSlider(QSlider::Vertical, hboxSliderBass);
	sliderBass->setTickmarks(QSlider::Right);
	sliderBass->setTickInterval(1);
	sliderBass->setPageStep(1);
	sliderBass->setMaxValue(15);
	sliderBass->setMinValue(1);
	connect(sliderBass, SIGNAL(valueChanged(int)), this, SLOT(sliderBassValueChanged(int)));
	
	QVBox *vboxSliderBassLabels = new QVBox(hboxSliderBass);
	
	lbBassTop = new QLabel(" +9dB", vboxSliderBassLabels);
	lbBassTop->setAlignment(Qt::AlignLeft | Qt::AlignTop);
	lbBassMid = new QLabel(" 0dB", vboxSliderBassLabels);
	lbBassMid->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
	lbBassBottom = new QLabel(" -6dB", vboxSliderBassLabels);
	lbBassBottom->setAlignment(Qt::AlignLeft | Qt::AlignBottom);

	QVBox *vboxBassCutOffFreq = new QVBox(gbBass);
	new QLabel("\n\nCut-Off:", vboxBassCutOffFreq);
	rbBassCutOffFreq130Hz = new QRadioButton("130 Hz", vboxBassCutOffFreq);
	connect(rbBassCutOffFreq130Hz, SIGNAL(toggled(bool)), this, SLOT(rbBassCutOffFreq130HzToggled(bool)));
	rbBassCutOffFreq200Hz = new QRadioButton("200 Hz", vboxBassCutOffFreq);
	connect(rbBassCutOffFreq200Hz, SIGNAL(toggled(bool)), this, SLOT(rbBassCutOffFreq200HzToggled(bool)));
	
	// Treble
	QVGroupBox *gbTreble = new QVGroupBox("Treble", hboxEqualizer);
	gbTreble->setAlignment(QGroupBox::AlignHCenter);
	
	cbTrebleEnabled = new QCheckBox("Enabled", gbTreble);
	connect(cbTrebleEnabled, SIGNAL(toggled(bool)), this, SLOT(cbTrebleEnabledToggled(bool)));
	
	QHBox *hboxSliderTreble = new QHBox(gbTreble);
	
	sliderTreble = new QSlider(QSlider::Vertical, hboxSliderTreble);
	sliderTreble->setTickmarks(QSlider::Right);
	sliderTreble->setTickInterval(1);
	sliderTreble->setPageStep(1);
	sliderTreble->setMaxValue(15);
	sliderTreble->setMinValue(1);
	connect(sliderTreble, SIGNAL(valueChanged(int)), this, SLOT(sliderTrebleValueChanged(int)));
	
	QVBox *vboxSliderTrebleLabels = new QVBox(hboxSliderTreble);
	QLabel *label = new QLabel(" +9dB", vboxSliderTrebleLabels);
	label->setAlignment(Qt::AlignLeft | Qt::AlignTop);

	label = new QLabel(" 0dB", vboxSliderTrebleLabels);
	label->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);

	label = new QLabel(" -6dB", vboxSliderTrebleLabels);
	label->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
	
	QVBox *vboxTrebleCutOffFreq = new QVBox(gbTreble);
	new QLabel("\n\nCut-Off:", vboxTrebleCutOffFreq);
	rbTrebleCutOffFreq4kHz = new QRadioButton("4 kHz", vboxTrebleCutOffFreq);
	connect(rbTrebleCutOffFreq4kHz, SIGNAL(toggled(bool)), this, SLOT(rbTrebleCutOffFreq4kHzToggled(bool)));
	rbTrebleCutOffFreq8kHz = new QRadioButton("8 kHz", vboxTrebleCutOffFreq);
	connect(rbTrebleCutOffFreq8kHz, SIGNAL(toggled(bool)), this, SLOT(rbTrebleCutOffFreq8kHzToggled(bool)));
	
	// 3D
	QVGroupBox *gb3D = new QVGroupBox("3D Stereo", hboxEqualizer);
	gb3D->setAlignment(QGroupBox::AlignHCenter);
	
	cb3DEnabled = new QCheckBox("Enabled", gb3D);
	connect(cb3DEnabled, SIGNAL(toggled(bool)), this, SLOT(cb3DEnabledToggled(bool)));
	
	QHBox *hboxSlider3D = new QHBox(gb3D);
	
	slider3D = new QSlider(QSlider::Vertical, hboxSlider3D);
	slider3D->setTickmarks(QSlider::Right);
	slider3D->setTickInterval(1);
	slider3D->setPageStep(1);
	slider3D->setMaxValue(15);
	slider3D->setMinValue(0);
	connect(slider3D, SIGNAL(valueChanged(int)), this, SLOT(slider3DValueChanged(int)));

	QVBox *vboxSlider3DLabels = new QVBox(hboxSlider3D);
	label = new QLabel(" 100%", vboxSlider3DLabels);
	label->setAlignment(Qt::AlignLeft | Qt::AlignTop);
	label = new QLabel(" 0%", vboxSlider3DLabels);
	label->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
	
	QHBox *hbox3DCutOffFreqs = new QHBox(gb3D);
	hbox3DCutOffFreqs->setSpacing(8);	
	
	QVBox *vbox3DLowerCutOffFreq = new QVBox(hbox3DCutOffFreqs);
	new QLabel("\nCut-Off\nLower:", vbox3DLowerCutOffFreq);
	rb3DLowerCutOffFreq200Hz = new QRadioButton("200 Hz", vbox3DLowerCutOffFreq);
	connect(rb3DLowerCutOffFreq200Hz, SIGNAL(toggled(bool)), this, SLOT(rb3DLowerCutOffFreq200HzToggled(bool)));
	rb3DLowerCutOffFreq500Hz = new QRadioButton("500 Hz", vbox3DLowerCutOffFreq);
	connect(rb3DLowerCutOffFreq500Hz, SIGNAL(toggled(bool)), this, SLOT(rb3DLowerCutOffFreq500HzToggled(bool)));

	QVBox *vbox3DUpperCutOffFreq = new QVBox(hbox3DCutOffFreqs);
	new QLabel("\n\nUpper:", vbox3DUpperCutOffFreq);
	rb3DUpperCutOffFreq1500Hz = new QRadioButton("1.5 kHz", vbox3DUpperCutOffFreq);
	connect(rb3DUpperCutOffFreq1500Hz, SIGNAL(toggled(bool)), this, SLOT(rb3DUpperCutOffFreq1500HzToggled(bool)));
	rb3DUpperCutOffFreq2200Hz = new QRadioButton("2.2 kHz", vbox3DUpperCutOffFreq);
	connect(rb3DUpperCutOffFreq2200Hz, SIGNAL(toggled(bool)), this, SLOT(rb3DUpperCutOffFreq2200HzToggled(bool)));
	
	// Set relative sizes of the three groupboxes in the top horizontal box
	hboxEqualizer->setStretchFactor(gbBass, 1);
	hboxEqualizer->setStretchFactor(gbTreble, 1);
	hboxEqualizer->setStretchFactor(gb3D, 2);
	
	// Add checkboxes for additional settings
	QHBox *hboxSpecialOptions = new QHBox(this);
	cbBassAdaptive = new QCheckBox("Adaptive Bass Boost", hboxSpecialOptions);
	connect(cbBassAdaptive, SIGNAL(toggled(bool)), this, SLOT(cbBassAdaptiveToggled(bool)));
	cb6dbAttenuation = new QCheckBox("-6 dB ATT/Prevent Clipping", hboxSpecialOptions);	
	connect(cb6dbAttenuation, SIGNAL(toggled(bool)), this, SLOT(cb6dbAttenuationToggled(bool)));
}

int WM8750Mixer::readProcValue(QCString key)
{
	FILE *fp = fopen(QCString("/proc/driver/wm8750/" + key), "r");

	if (fp)
	{
		int value;
		fscanf(fp, "%d", &value);
		fclose(fp);
		return value;
	}
	else
		return 0;
}

void WM8750Mixer::writeProcValue(QCString key, int value)
{
	if (updating_) return;
	
	FILE *fp = fopen(QCString("/proc/driver/wm8750/" + key), "w");
	
	if (fp)
	{
		fprintf(fp, "%d", value);
		fclose(fp);
	}
	updateValues(); 
}

void WM8750Mixer::updateValues()
{
	int value;
	
	updating_ = true;
	
	int autodetect = readProcValue("headphone_autodetect");
	value = readProcValue("headphone_state");
	
	pbOutputAuto->setOn(autodetect == 1);
	pbOutputInternalSpeaker->setOn(autodetect == 0 && value == 0);
	pbOutputHeadphone->setOn(autodetect == 0 && value == 1);
	
	cb6dbAttenuation->setChecked(readProcValue("dac_6db_attenuation") == 1);
	
	value = readProcValue("bass_adaptive_boost");
	cbBassAdaptive->setChecked(value == 1);
	lbBassTop->setText(value == 1 ? " 100%" : " +9dB");
	lbBassMid->setText(value == 1 ? "" : " 0dB");
	lbBassBottom->setText(value == 1 ? " 0%" : " -6dB");

	value = readProcValue("bass_intensity");
	sliderBass->setValue(16 - value);
	cbBassEnabled->setChecked(value != 0);
	
	value = readProcValue("bass_cutoff_high");
	rbBassCutOffFreq130Hz->setChecked(value == 0);
	rbBassCutOffFreq200Hz->setChecked(value == 1);
		
	value = readProcValue("treble_intensity");
	sliderTreble->setValue(16 - value);
	cbTrebleEnabled->setChecked(value != 0);

	value = readProcValue("treble_cutoff_low");
	rbTrebleCutOffFreq8kHz->setChecked(value == 0);
	rbTrebleCutOffFreq4kHz->setChecked(value == 1);
	
	slider3D->setValue(15 - readProcValue("s3denh_depth"));
	cb3DEnabled->setChecked(readProcValue("s3denh_enabled") == 1);
	
	value = readProcValue("s3denh_lower_cutoff_high");
	rb3DLowerCutOffFreq200Hz->setChecked(value == 0);
	rb3DLowerCutOffFreq500Hz->setChecked(value == 1);
	
	value = readProcValue("s3denh_upper_cutoff_low");
	rb3DUpperCutOffFreq2200Hz->setChecked(value == 0);
	rb3DUpperCutOffFreq1500Hz->setChecked(value == 1);
	
	updating_ = false;
}

void WM8750Mixer::about()
{
	QMessageBox mb("About WM8750Mixer", 
			"WM8750Mixer 0.9\n"
			"Copyright (C) 2007 by Andre Beckedorf\n"
			"evilJazz@katastrophos.net\n\n"
			"This software is licensed under the terms of\n"
			"the GNU Public License version 2.\n\n"
			"For more information please visit:\n"
			"http://katastrophos.net/zaurus/sources/wm8750mixer", QMessageBox::NoIcon, 1, 0, 0);
	mb.exec(); 
}
