/*
 * Copyright (C) 2007 Andre Beckedorf <evilJazz _AT_ katastrophos _DOT_ net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef WM8750Mixer_H_INCLUDED
#define WM8750Mixer_H_INCLUDED

#include <qmainwindow.h>
#include <qstring.h>

class QLabel;
class QProgressBar;
class QTimer;
class QListView;
class QCheckBox;
class QRadioButton;
class QPushButton;
class QSlider;
class QVGroupBox;
class QWidgetStack;

class WM8750Mixer: public QMainWindow
{
	Q_OBJECT

public:
	WM8750Mixer();
	virtual ~WM8750Mixer();

protected:
	void closeEvent(QCloseEvent*);
	
private:
	QCheckBox *cbBassEnabled;
	QSlider *sliderBass;
	QLabel *lbBassTop, *lbBassMid, *lbBassBottom;
	QRadioButton *rbBassCutOffFreq130Hz, *rbBassCutOffFreq200Hz;
	
	QCheckBox *cbTrebleEnabled;
	QSlider *sliderTreble;
	QRadioButton *rbTrebleCutOffFreq4kHz, *rbTrebleCutOffFreq8kHz;
	
	QCheckBox *cb3DEnabled;
	QSlider *slider3D;
	
	QRadioButton *rb3DLowerCutOffFreq200Hz, *rb3DLowerCutOffFreq500Hz;
	QRadioButton *rb3DUpperCutOffFreq1500Hz, *rb3DUpperCutOffFreq2200Hz;
	
	QCheckBox *cbBassAdaptive, *cb6dbAttenuation;
	
	QPushButton *pbOutputAuto, *pbOutputInternalSpeaker, *pbOutputHeadphone;
			
	bool updating_;
	
	void updateWidgets();
	
	int readProcValue(QCString key);
	void writeProcValue(QCString key, int value);	
	void updateValues();
    
private slots:
	void about();
	
	void sliderBassValueChanged(int value) { writeProcValue("bass_intensity", 16 - value); }
	void sliderTrebleValueChanged(int value) { writeProcValue("treble_intensity", 16 - value); }
	void slider3DValueChanged(int value) { writeProcValue("s3denh_depth", 15 - value); }
	void cbBassEnabledToggled(bool value) { writeProcValue("bass_intensity", (value ? 8 : 0)); }
	void cbTrebleEnabledToggled(bool value) { writeProcValue("treble_intensity", (value ? 8 : 0)); }
	void cb3DEnabledToggled(bool value) { writeProcValue("s3denh_enabled", (value ? 1 : 0)); }
	void cbBassAdaptiveToggled(bool value) { writeProcValue("bass_adaptive_boost", (value ? 1 : 0)); }
	void cb6dbAttenuationToggled(bool value) { writeProcValue("dac_6db_attenuation", (value ? 1 : 0)); }
	
	void rbBassCutOffFreq130HzToggled(bool) { writeProcValue("bass_cutoff_high", 0); }
	void rbBassCutOffFreq200HzToggled(bool) { writeProcValue("bass_cutoff_high", 1); }
	void rbTrebleCutOffFreq4kHzToggled(bool) { writeProcValue("treble_cutoff_low", 1); }
	void rbTrebleCutOffFreq8kHzToggled(bool) { writeProcValue("treble_cutoff_low", 0); }
	
	void rb3DLowerCutOffFreq200HzToggled(bool) { writeProcValue("s3denh_lower_cutoff_high", 0); }
	void rb3DLowerCutOffFreq500HzToggled(bool) { writeProcValue("s3denh_lower_cutoff_high", 1); }
	void rb3DUpperCutOffFreq1500HzToggled(bool) { writeProcValue("s3denh_upper_cutoff_low", 1); }
	void rb3DUpperCutOffFreq2200HzToggled(bool) { writeProcValue("s3denh_upper_cutoff_low", 0); }
	
	void pbOutputAutoToggled(bool) { writeProcValue("headphone_autodetect", 1); }
	void pbOutputInternalSpeakerToggled(bool) { writeProcValue("headphone_autodetect", 0); writeProcValue("headphone_state", 0); }
	void pbOutputHeadphoneToggled(bool) { writeProcValue("headphone_autodetect", 0); writeProcValue("headphone_state", 1); }
};

#endif // WM8750Mixer_H_INCLUDED
